/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: battery-meter
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-09-02, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  *       - shows nwid
  *       - nicer output
  * -------------------------------------------------------------------
  * TODO:
  *       - use icons
  */
package jaugment2.wlanmonitor;

     //JDK
import javax.swing.*;
import java.awt.*;
import java.util.GregorianCalendar;
import net.jini.core.event.*;
import jaugment2.helpers.Logger;
import jaugment2.battery.BatteryService.BatteryServiceEvent;
     //JINI
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;
import net.jini.core.lease.Lease;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;


/**
 * Small dockin for the SwingUI showing wireless link-quality
 */
public class WLanMonitorDockin extends JPanel implements RemoteEventListener
{
//  JLabel jLabel1 = new JLabel();
  JTextArea jLabel1 = new JTextArea
  ();
  private static final boolean debugMessages = true;

  private WLanMonitorProxyIF proxy = null;
  BorderLayout borderLayout1 = new BorderLayout();

  public WLanMonitorDockin(WLanMonitorProxyIF p)
  {
   this.proxy = p;
    try
    {
      jbInit();
      p.addListener(this, null, WLanMonitorProxyIF.WLanMonitorEvent.EVID_LINKLEVELUPDATED);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

 public void setFont(Font f)
 {
  if(f == null)
    return;
  if(jLabel1 == null)
    return;
  Font f2 = new Font(f.getFontName(), f.getStyle(), (int)(f.getSize()*0.75));
  super.setFont(f2);
  jLabel1.setFont(f2);
 }

  private String f2s(float f)
  {
   String s = ""+f;
   if(s.endsWith(".0"))
     s = s.substring(0,s.length()-2);
   return(s);
  }


  private void jbInit() throws Exception
  {
   if(debugMessages)Logger.debug.println("DEBUG: WLanMonitorDockin.jbInit()...");
    jLabel1.setFont(new java.awt.Font("Dialog", 1, 10));
    jLabel1.setText("any:??\n??/??");//TODO: graphical
/*    this.setMaximumSize(new Dimension(79, 30));
    this.setMinimumSize(new Dimension(59, 20));
    this.setPreferredSize(new Dimension(69, 20));*/
    this.setMaximumSize(new Dimension(79, 39));
    this.setMinimumSize(new Dimension(59, 30));
    this.setPreferredSize(new Dimension(69, 30));
    this.setLayout(borderLayout1);
    this.add(jLabel1, BorderLayout.CENTER);

    try
      {
       float level = proxy.getLink();
       float noise = proxy.getNoise();
       float link = proxy.getLink();
       String nwid = proxy.getNWID();
       jLabel1.setRows(2);
       jLabel1.setEditable(false);
       jLabel1.setMargin(new java.awt.Insets(0,0,0,0));
       jLabel1.setBackground(this.getBackground());
       jLabel1.setText(nwid+":"+f2s(level)+"\n"+f2s(link)+"/"+f2s(noise));
       jLabel1.setForeground( ((level<5)?Color.red:((level<20)?Color.yellow:Color.black)) ) ;
      }
    catch(Exception e)
      {
       e.printStackTrace(Logger.err);
      }
  }



  public void notify(RemoteEvent p0) throws UnknownEventException//, RemoteException
  {
   if(!(p0 instanceof WLanMonitorProxyIF.WLanMonitorEvent))
     throw(new UnknownEventException("expecting WLanMonitorEvent") );

    WLanMonitorProxyIF.WLanMonitorEvent mevt = (WLanMonitorProxyIF.WLanMonitorEvent)p0;
//    jLabel1.setText(mevt.nwid+":"+mevt.level+"/"+mevt.link+"/"+mevt.noise);
    jLabel1.setText(mevt.nwid+":"+f2s(mevt.level)+"\n"+f2s(mevt.link)+"/"+f2s(mevt.noise));
    jLabel1.setForeground( ((mevt.level<5)?Color.red:((mevt.level<20)?Color.yellow:Color.black)) ) ;
  }

}