/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: WLan-monitor
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-09-02, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  *       - it works
  * -------------------------------------------------------------------
  * TODO:
  *       - draw an icon for this service
  */

package jaugment2.wlanmonitor;


  //JDK
//import java.rmi.server.UnicastRemoteObject;
import java.rmi.RemoteException;
import java.rmi.MarshalledObject;
import java.util.LinkedList;
import java.io.FileInputStream;
  //JINI
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;
import net.jini.core.lease.Lease;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.UnknownEventException;
  //serviceui-draft-standart
import net.jini.lookup.entry.UIDescriptor;
  //project
import jaugment2.jini.simpleService.VerySimpleService;
import jaugment2.helpers.Logger;
import jaugment2.jini.SimpleLandlord;
import jaugment2.jini.SimpleLandlord.LeaseListener;
import jaugment2.jini.SimpleLandlord.EventRegistration;

//helper-mark for configure to determine that rmic needs to be executed for this file
//rmimakehelper


/**
 * Title:        WLan-Monitor
 * Description:  Monitors wireless-lan link-quality
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Marcus Wolschon Suran@gmx.net
 * @version 1.0
 */

public class WLanMonitorService extends VerySimpleService implements WLanMonitorProxyIF
{
 private static final boolean debugThis = true;

  public WLanMonitorService() throws java.rmi.RemoteException , java.io.IOException
  {
   if(debugThis)Logger.debug.println("DEBUG: WLanMonitorService starting up...");
   landLord = new SimpleLandlord(simpleLeaselistener);
   init(this, jaugment2.wlanmonitor.WLanMonitorProxyIF.class);


      Thread update = new Thread()
       {
        public void run()
        {
         try
           {
            while(true)
              {
               update();
               String sleep = System.getProperty("jaugment2.wlanmonitor.WLanMonitorService.updateintervall");
               sleep(sleep==null?10000:Integer.parseInt(sleep));

              }
           }
         catch(Exception e)
           {
            e.printStackTrace(Logger.err);
           }
        }
       };
      update.setDaemon(true);
      update.start();

  }


  public String getComment()
  {return("monitors wireless-lan link-quality");};
  public String getCreator()
  {return("Marcus Wolschon <Suran@gmx.net>");};
  public UIDescriptor getUIDescriptor()
  {
   if(debugThis)Logger.debug.println("DEBUG: WLanMonitorService.getUIDescriptor() called...");
   return(WLanMonitorSwingRepresentation.getUIDescriptor());
  };



 public float getLink() throws RemoteException
 {
  return(link);
 }
 public float getNoise() throws RemoteException
 {
  return(noise);
 }
 public float getLevel() throws RemoteException
 {
  return(level);
 }
 public String getNWID() throws RemoteException
 {
  return(nwid);
 }

 private float level = 0.0f;
 private float link  = 0.0f;
 private float noise = 0.0f;
 private String nwid = "";


 private String readLine(FileInputStream fis) throws java.io.IOException
 {
  StringBuffer sb = new StringBuffer();
  int c = 0;
  while(((c = fis.read()) >0 ) && (c == 10))  ;
  while(((c = fis.read()) >0 ) && (c == 13))  ;
  while(((c = fis.read()) >0 ) && (c != 10) && (c != 13))
    {
     sb = sb.append((char)c);
    }
  return(sb.toString());
 };


 void update()
 {
  FileInputStream fis = null;
  try
    {
     fis = new FileInputStream("/proc/net/wireless");
     //TODO
     String line = readLine(fis);
     if(debugThis)Logger.debug.println("DEBUG: WLanMonitorService.update() line0=\""+line+"\"");
            line = readLine(fis);
     if(debugThis)Logger.debug.println("DEBUG: WLanMonitorService.update() line1=\""+line+"\"");
            line = readLine(fis);
     if(debugThis)Logger.debug.println("DEBUG: WLanMonitorService.update() line2=\""+line+"\"");
     if(line.length() > 0)
       {
        java.util.StringTokenizer st = new java.util.StringTokenizer(line, " ", false);
        st.nextToken();
        st.nextToken();
        String links  =         st.nextToken();
        String levels =         st.nextToken();
        String noises =         st.nextToken();
               nwid   =         st.nextToken();

        fis.close();

        link  = Float.parseFloat(links+"0"  )+0.0f;
        level = Float.parseFloat(levels+"0" )+0.0f;
        noise = Float.parseFloat(noises+"0" )+0.0f;

       }
     else
        fis.close();
     fireEvent(WLanMonitorProxyIF.WLanMonitorEvent.EVID_LINKLEVELUPDATED);
    }
  catch(Exception e)
    {
     if(fis != null)
        try{fis.close();}
        catch(Exception e2){}
     e.printStackTrace(Logger.err);
    }
 }




























   private LinkedList listenerRegistrations[] = new LinkedList[]{new LinkedList(),new LinkedList()};

  /**
   * Helper-class for our Landlord.
   */
  private LeaseListener simpleLeaselistener = new LeaseListener()
  {
   public void leaseCanceled(int type, Object reg, int newcount)
   {
//    EventRegistration reg2 = (EventRegistration)reg;
    //TODO
   };
  };

  private SimpleLandlord landLord;

  private void fireEvent(int type)
  {
   if(debugThis)Logger.debug.println("DEBUG: [WLanMonitorService:fireEvent] I have "+listenerRegistrations[type].size()+" registrations");
   for(int i=0; i<listenerRegistrations[type].size(); i++)
     {
      try
        {                     //TODO: do this asyncronously
         EventRegistration entry = (EventRegistration)listenerRegistrations[type].get(i);
         if(debugThis)Logger.debug.println("DEBUG: [WLanMonitorService:fireEvent] delivering event type="+type+" subscribedFor="+entry.subscribedFor+" entry.subscribedFor & type="+(entry.subscribedFor & type));
         if( (entry.subscribedFor & type) != 0)
         entry.listener.notify(
                               new WLanMonitorProxyIF.WLanMonitorEvent(getProxy(),
                                                     type,
                                                     entry.seq++,
                                                     entry.handback,
                                                     level,
                                                     link,
                                                     noise,
                                                     nwid
                                                    )
                              );
        }
      catch(RemoteException ex)
        {
         Logger.err.println("ERROR: [WLanMonitorService:fireEvent] could not deliver event");
         ex.printStackTrace(Logger.err);
        }
      catch(UnknownEventException ex)
        {
         Logger.err.println("ERROR: [WLanMonitorService:fireEvent] could not deliver event - the receiver does not know it");
         ex.printStackTrace(Logger.err);
        }
     }
  };

  /** Subscribe for events
    */
  public Lease addListener(RemoteEventListener listener, MarshalledObject handback, int registerfor) throws RemoteException, LeaseDeniedException, IllegalStateException
  {
   if(listener==null)
     return(null);

   try
     {
      EventRegistration reg = new EventRegistration(listener, handback);
      Lease   l = landLord.grantLease(0, reg, 2*60*10);

      if(l != null)
        {
          // tokenize the expression
         if(debugThis)Logger.debug.println("DEBUG: [WLanMonitorService] addListener expr=\""+registerfor+"\"");

         for(int i=1; i<(2*registerfor) && (i<listenerRegistrations.length); i=i*2)
            if((i & registerfor) == i)
             {
              listenerRegistrations[i].add(reg);
             };
        };

      return(l);
     }
   catch(Exception ex)
     {
      ex.printStackTrace(Logger.err);
     }
   return(null);
  };









}